//  CoreDataCookingAppDelegate.m
//  CoreDataCooking
//
//  Created by ?? ?? on 05/05/26.
//  Copyright __MyCompanyName__ 2005 . All rights reserved.

#import "CoreDataCookingAppDelegate.h"
#import "CoreDataAdditions.h"

@implementation CoreDataCookingAppDelegate

- (id)init
{
    [super init];
    predicateString = @"name like '*'";
    [self createTestData];
    return self;
}

- (void)createTestData
{
    id obj, entity, request, objects, type, company;
    
    entity = [NSEntityDescription entityForName:@"Product"
                         inManagedObjectContext:[self managedObjectContext]];
    request = [[[NSFetchRequest alloc] init] autorelease];
    [request setEntity:entity];
    objects = [[self managedObjectContext] executeFetchRequest:request error:nil];
    if ([objects count] > 0) return;
    
    obj = [NSEntityDescription insertNewObjectForEntityForName:@"ProductType"
                inManagedObjectContext:[self managedObjectContext]];
    [obj setValue:@"Hardware" forKey:@"name"];

    type = [NSEntityDescription insertNewObjectForEntityForName:@"ProductType"
                                        inManagedObjectContext:[self managedObjectContext]];
    [type setValue:@"Software" forKey:@"name"];
    
    obj = [NSEntityDescription insertNewObjectForEntityForName:@"ProductType"
                                        inManagedObjectContext:[self managedObjectContext]];
    [obj setValue:@"Services" forKey:@"name"];
    
    company = [NSEntityDescription insertNewObjectForEntityForName:@"Company"
                                        inManagedObjectContext:[self managedObjectContext]];
    [company setValue:@"Apple" forKey:@"name"];
    
    obj = [NSEntityDescription insertNewObjectForEntityForName:@"Product"
                                        inManagedObjectContext:[self managedObjectContext]];
    [obj setValue:@"WebObjects" forKey:@"name"];
    [obj setValue:type forKey:@"productType"];
    [obj setValue:company forKey:@"company"];
    
    obj = [NSEntityDescription insertNewObjectForEntityForName:@"Product"
                                        inManagedObjectContext:[self managedObjectContext]];
    [obj setValue:@"Mac OS X 10.4" forKey:@"name"];
    [obj setValue:type forKey:@"productType"];
    [obj setValue:company forKey:@"company"];
    
    obj = [NSEntityDescription insertNewObjectForEntityForName:@"Product"
                                        inManagedObjectContext:[self managedObjectContext]];
    [obj setValue:@"iWork" forKey:@"name"];
    [obj setValue:type forKey:@"productType"];
    [obj setValue:company forKey:@"company"];
    
    obj = [NSEntityDescription insertNewObjectForEntityForName:@"Product"
                                        inManagedObjectContext:[self managedObjectContext]];
    [obj setValue:@"Keynote" forKey:@"name"];
    [obj setValue:type forKey:@"productType"];
    [obj setValue:company forKey:@"company"];
    
    [[self managedObjectContext] save:nil];
}

- (void)dealloc
{
    [predicateString release];
    [super dealloc];
}

- (NSManagedObjectModel *)managedObjectModel {
    if (managedObjectModel) return managedObjectModel;
	
	NSMutableSet *allBundles = [[NSMutableSet alloc] init];
	[allBundles addObject: [NSBundle mainBundle]];
	[allBundles addObjectsFromArray: [NSBundle allFrameworks]];
    
    managedObjectModel = [[NSManagedObjectModel mergedModelFromBundles: [allBundles allObjects]] retain];
    [allBundles release];
    
    return managedObjectModel;
}

/* Change this path/code to point to your App's data store. */
- (NSString *)applicationSupportFolder {
    NSString *applicationSupportFolder = nil;
    FSRef foundRef;
    OSErr err = FSFindFolder(kUserDomain, kApplicationSupportFolderType, kDontCreateFolder, &foundRef);
    if (err != noErr) {
        NSRunAlertPanel(@"Alert", @"Can't find application support folder", @"Quit", nil, nil);
        [[NSApplication sharedApplication] terminate:self];
    } else {
        unsigned char path[1024];
        FSRefMakePath(&foundRef, path, sizeof(path));
        applicationSupportFolder = [NSString stringWithUTF8String:(char *)path];
        applicationSupportFolder = [applicationSupportFolder stringByAppendingPathComponent:@"CoreDataCooking"];
    }
    return applicationSupportFolder;
}

- (NSManagedObjectContext *) managedObjectContext {
    NSError *error;
    NSString *applicationSupportFolder = nil;
    NSURL *url;
    NSFileManager *fileManager;
    NSPersistentStoreCoordinator *coordinator;
    
    if (managedObjectContext) {
        return managedObjectContext;
    }
    
    fileManager = [NSFileManager defaultManager];
    applicationSupportFolder = [self applicationSupportFolder];
    if ( ![fileManager fileExistsAtPath:applicationSupportFolder isDirectory:NULL] ) {
        [fileManager createDirectoryAtPath:applicationSupportFolder attributes:nil];
    }
    
    url = [NSURL fileURLWithPath: [applicationSupportFolder stringByAppendingPathComponent: @"CoreDataCooking.xml"]];
    coordinator = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel: [self managedObjectModel]];
    if ([coordinator addPersistentStoreWithType:NSXMLStoreType configuration:nil URL:url options:nil error:&error]){
        managedObjectContext = [[NSManagedObjectContext alloc] init];
        [managedObjectContext setPersistentStoreCoordinator: coordinator];
    } else {
        [[NSApplication sharedApplication] presentError:error];
    }    
    [coordinator release];
    
    return managedObjectContext;
}

- (NSUndoManager *)windowWillReturnUndoManager:(NSWindow *)window {
    return [[self managedObjectContext] undoManager];
}

- (IBAction) saveAction:(id)sender {
    NSError *error = nil;
    NSLog(@"==> save");
    [self logPendingObjects];
    NSLog(@"* save");
    
    if (![[self managedObjectContext] save:&error]) {
        [[NSApplication sharedApplication] presentError:error];
    }
    
    [self logPendingObjects];
    NSLog(@"<== save");
}

- (IBAction)resetAction:(id)sender
{
    NSLog(@"==> reset");
    [self logPendingObjects];
    NSLog(@"* reset");
    
    // 更新待ちのオブジェクトがすべて元に戻る
    [[self managedObjectContext] reset];
    
    [self logPendingObjects];
    NSLog(@"<== reset");
}

- (IBAction)fetchAction:sender
{
    NSFetchRequest *request;
    NSPredicate *pred;
    NSEntityDescription *entity;
    NSArray *fetchedObjects;
    
    NSLog(@"==> fetch");
    if (predicateString) {
        // 条件
        pred = [NSPredicate predicateWithFormat:predicateString];

        // エンティティ
        entity = [NSEntityDescription entityForName:@"Product"
                    inManagedObjectContext:[self managedObjectContext]];
        
        // 要求に上記をセット
        request = [[[NSFetchRequest alloc] init] autorelease];
        [request setEntity:entity];
        [request setPredicate:pred];
        
        // フェッチ
        fetchedObjects = [[self managedObjectContext] executeFetchRequest:request error:nil];
        
        // おまけメソッドを使うとこれ一行になる
        /*
        fetchedObjects = [[self managedObjectContext]
                            fetchWithEntityName:@"Product"
                                          error:nil
                            predicateWithFormat:predicateString];
        */
        
        NSLog(@"* fetched %d: %@", [fetchedObjects count], [[fetchedObjects valueForKey:@"name"] description]);
    }
    NSLog(@"<== fetch");
}

- (IBAction)insertAction:(id)sender {
    id objects;
    NSManagedObject *productType, *company;
    NSFetchRequest *request;
    NSEntityDescription *entity;
    NSString *productName;
    
    NSLog(@"==> insert");
    [self logPendingObjects];

    // 名前は適当
    productName = @"New Product";
    
    // 適当にproduct typeをフェッチ
    // NSPredicateを省略するとすべてデータを取得する
    request = [[[NSFetchRequest alloc] init] autorelease];
    entity = [NSEntityDescription entityForName:@"ProductType"
                inManagedObjectContext:[self managedObjectContext]];
    [request setEntity:entity];
    [request setFetchLimit:1]; // 1つだけオブジェクトを取得
    objects = [[self managedObjectContext] executeFetchRequest:request error:nil];
    productType = [objects lastObject];
    NSLog(@"* fetched product type: %@", [productType valueForKey:@"name"]);

    // 適当にcompanyをフェッチ
    request = [[[NSFetchRequest alloc] init] autorelease];
    entity = [NSEntityDescription entityForName:@"Company"
                         inManagedObjectContext:[self managedObjectContext]];
    [request setEntity:entity];
    [request setFetchLimit:1]; // 1つだけオブジェクトを取得
    objects = [[self managedObjectContext] executeFetchRequest:request error:nil];
    company = [objects lastObject];
    NSLog(@"* fetched company: %@", [company valueForKey:@"name"]);
    
    // オブジェクトを生成し、managed object contextに追加
    // 保存するまでは一時的なオブジェクトとして保持される
    id obj = [NSEntityDescription insertNewObjectForEntityForName:@"Product"
                                  inManagedObjectContext:[self managedObjectContext]];
    [obj setValue:productName forKey:@"name"];
    [obj setValue:productType forKey:@"productType"];
    [obj setValue:company forKey:@"company"];
    
    NSLog(@"* inserted: %@", [obj description]);
    [self logPendingObjects];
    NSLog(@"<== insert");
}

- (IBAction)deleteAction:sender
{
    NSFetchRequest *request;
    NSEntityDescription *entity;
    NSArray *objects;
    id e, obj;
    
    NSLog(@"==> delete");
    [self logPendingObjects];
    
    entity = [NSEntityDescription entityForName:@"Product"
                    inManagedObjectContext:[self managedObjectContext]];
    request = [[[NSFetchRequest alloc] init] autorelease];
    [request setEntity:entity];
    objects = [[self managedObjectContext] executeFetchRequest:request error:nil];        
    
    // 全部削除
    // ただし、保存するまでは実際に削除されない
    e = [objects objectEnumerator];
    while (obj = [e nextObject]) {
        [[self managedObjectContext] deleteObject:obj];
    }
    
    [self logPendingObjects];
    NSLog(@"<== delete");
}

- (IBAction)updateAction:sender
{
    NSFetchRequest *request;
    NSEntityDescription *entity;
    NSArray *objects;
    id obj;
    
    NSLog(@"==> update");
    [self logPendingObjects];
    
    entity = [NSEntityDescription entityForName:@"Product"
                         inManagedObjectContext:[self managedObjectContext]];
    request = [[[NSFetchRequest alloc] init] autorelease];
    [request setEntity:entity];
    objects = [[self managedObjectContext] executeFetchRequest:request error:nil];        
    
    // オブジェクトを変更
    // 変更すると更新待ちになり、updatedObjectsに登録される
    obj = [objects lastObject];
    [obj setValue:[[NSDate date] description] forKey:@"name"];
    NSLog(@"* updated: %@", [obj description]);
    
    [self logPendingObjects];
    NSLog(@"<== update");
}

- (void)logPendingObjects
{
    NSLog(@"* pending objects - registered: %d, inserted: %d, updated: %d, deleted: %d",
          [[[self managedObjectContext] registeredObjects] count],
          [[[self managedObjectContext] insertedObjects] count],
          [[[self managedObjectContext] updatedObjects] count],
          [[[self managedObjectContext] deletedObjects] count]);
}

- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender {
    NSError *error;
    NSManagedObjectContext *context;
    int reply = NSTerminateNow;
    
    context = [self managedObjectContext];
    if (context != nil) {
        if ([context commitEditing]) {
            if (![context save:&error]) {
				
				// This default error handling implementation should be changed to make sure the error presented includes application specific error recovery. For now, simply display 2 panels.
                BOOL errorResult = [[NSApplication sharedApplication] presentError:error];
				
				if (errorResult == YES) { // Then the error was handled
					reply = NSTerminateCancel;
				} else {
					
					// Error handling wasn't implemented. Fall back to displaying a "quit anyway" panel.
					int alertReturn = NSRunAlertPanel(nil, @"Could not save changes while quitting. Quit anyway?" , @"Quit anyway", @"Cancel", nil);
					if (alertReturn == NSAlertAlternateReturn) {
						reply = NSTerminateCancel;	
					}
				}
            }
        } else {
            reply = NSTerminateCancel;
        }
    }
    return reply;
}

@end
